﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using System.Drawing.Drawing2D;

namespace howto_colored_sierpinski_pentagon
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        // The Pentagon objects.
        private List<Pentagon> Pentagons = new List<Pentagon>();

        // Set the selected color.
        private void colorLabel_Click(object sender, EventArgs e)
        {
            Label clicked_label = sender as Label;
            lblSelected.BackColor = clicked_label.BackColor;
        }

        // Redraw.
        private void nudDepth_ValueChanged(object sender, EventArgs e)
        {
            MakePentagons();
        }

        // Make the Pentagon objects and redraw.
        private void MakePentagons()
        {
            // Build a new list of Pentagons.
            int depth = (int)nudDepth.Value;
            PointF center = new PointF(
                picPentagon.ClientSize.Width / 2,
                picPentagon.ClientSize.Height / 2);
            float radius = (float)Math.Min(center.X, center.Y);
            Pentagons = new List<Pentagon>();
            MakePentagons(depth, center, radius);

            // Redraw.
            picPentagon.Refresh();
        }

        // Scale factor for moving to smaller pentagons.
        private float size_scale = (float)(1.0 / (2.0 * (1 + Math.Cos(Math.PI / 180 * 72))));

        // Recursively generate the Pentagons.
        private void MakePentagons(int depth, PointF center, float radius)
        {
            // If we are done recursing, add a new Pentagon to the list.
            if (depth <= 0)
            {
                // Find the pentagon's corners.
                Pentagons.Add(new Pentagon(
                    GetPentagonPoints(center, radius),
                    lblSelected.BackColor));
            }
            else
            {
                // Find the smaller pentagons' centers.
                float d = radius - radius * size_scale;
                PointF[] centers = GetPentagonPoints(center, d);

                // Recursively draw the smaller pentagons.
                foreach (PointF point in centers)
                {
                    MakePentagons(depth - 1, point, radius * size_scale);
                }
            }
        }

        // Find the pentagon's corners.
        private PointF[] GetPentagonPoints(PointF center, float radius)
        {
            PointF[] points = new PointF[5];
            double theta = -Math.PI / 2.0;
            double dtheta = 2.0 * Math.PI / 5.0;
            for (int i = 0; i < 5; i++)
            {
                points[i] = new PointF(
                    center.X + (float)(radius * Math.Cos(theta)),
                    center.Y + (float)(radius * Math.Sin(theta)));
                theta += dtheta;
            }
            return points;
        }

        // Draw.
        private void picPentagon_Paint(object sender, PaintEventArgs e)
        {
            e.Graphics.Clear(picPentagon.BackColor);
            e.Graphics.SmoothingMode = SmoothingMode.AntiAlias;
            foreach (Pentagon pentagon in Pentagons)
                pentagon.Draw(e.Graphics);
        }

        // Color the clicked Pentagon.
        private void picPentagon_MouseClick(object sender, MouseEventArgs e)
        {
            // Get the clicked point.
            PointF point = e.Location;

            // Find the clicked Pentagon.
            foreach (Pentagon pentagon in Pentagons)
            {
                if (pentagon.Contains(point))
                {
                    // Color this pentagon and redraw.
                    pentagon.FillColor = lblSelected.BackColor;
                    picPentagon.Refresh();
                    return;
                }
            }
        }

        // Draw the initial pentagon.
        private void Form1_Load(object sender, EventArgs e)
        {
            MakePentagons();
        }
    }
}
